import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import massim.rmi.StringMapServer;
import massim.rmi.XMLDocumentServer;

import org.w3c.dom.Document;

public class ServerMonitorThread extends Thread {
	
	private String host;
	private String port;
	private String service;
	private int sleeptime;
	private boolean simulationRunning = false;
	private Map<String, String> simulationStatus;
	private Document xmlDoc;
	private TransformerFactory transformerfactory = TransformerFactory.newInstance();
	private boolean xmlService = false;
	private boolean mapService = false;
	
	/**
	 * creates a new ServerMonitorThread
	 * @param host RMI-host
	 * @param port RMI-port
	 * @param service RMI-servicename
	 * @param sleeptime Threadsleeptime
	 */
	public ServerMonitorThread(String host, String port, String service, int sleeptime)
	{
		this.host = host;
		this.port = port;
		this.service = service;
		
		if(sleeptime < 0) this.sleeptime = sleeptime * (-1);
		else this.sleeptime = sleeptime;
	}
	
	/**
	 * The run-method of the thread. 
	 */
	public void run()
	{
		while (! isInterrupted()) 
		{
			getRMIObject(host, port, service);
			
			if( simulationRunning == true )
			{
				System.out.println("########################################\n");
				
				//test if data of a Hashmap or a xml-Document should be printed on standard-out
				if(mapService == true)
					printStatus(simulationStatus);
				else if(xmlService == true) 
					printXMLDoc(xmlDoc);
			}
						
			try
		    {
				System.out.println("########################################");
				System.out.println("\nSleeping " + (sleeptime/1000) +" seconds...\n");
				System.out.println("########################################\n");
				Thread.sleep( sleeptime );
		    }
			catch ( InterruptedException e )
		    {
				interrupt();
				System.out.println("Shutting down...\n");
		    }
		}
	}
	
	/**
	 * This Method tried to get an object via RMI
	 * @param host RMI-host
	 * @param port RMI-port
	 * @param service RMI-servicename
	 */
	private void getRMIObject(String host, String port, String service)
	{
		try
		{
			Object o = null;
			
			simulationRunning = true;
			xmlService = false;
			mapService = false;
						
			String url = "rmi://" + host + ":" + port + "/" + service; //create the url 
			o = Naming.lookup(url);
							
			//test if o is an instance of massim.rmi.StringMapServer or massim.rmi.XMLDocumentServer
			if( o instanceof massim.rmi.StringMapServer) {
				StringMapServer server_state = (StringMapServer)Naming.lookup(url);
				simulationStatus = (Map<String,String>)server_state.get();
				mapService = true;
			}
			else if( o instanceof massim.rmi.XMLDocumentServer) {	
				XMLDocumentServer server_state = (XMLDocumentServer)Naming.lookup(url);
				xmlDoc = (Document)server_state.getXMLDocument();
				xmlService = true;
			}
		}
		catch (MalformedURLException e) 
		{
			//e.printStackTrace();
			System.err.println("A malformed URL has occurred or no legal protocol could be found.\n");
			simulationRunning = false;
		} 
		catch (RemoteException e) 
		{
			//e.printStackTrace();
			System.err.println("Currently no simulation running on " + host + " " + port +"...\n");
			//System.err.println("Is the classpath set correctly?\n");
			simulationRunning = false;
		} 
		catch (NotBoundException e) 
		{
			//e.printStackTrace();
			System.err.println("Currently no simulation running on " + host + " " + port +"...\n");
			simulationRunning = false;
		}
	}
	
	/**
	 * This method prints data of a Hashmap on standard-out
	 * @param statsMap Hashmap with data
	 */
	private void printStatus(Map<String, String> statsMap)
	{
		Set<String> e = statsMap.keySet();
		Iterator<String> b = e.iterator();
		int i = statsMap.size(); 
		String keys[] = new String[i];
		
		i=0;
		while (b.hasNext())
		{
			keys[i] = b.next();  //save all keys in array keys[]
			i++;
		}
		List<String> l = Arrays.asList(keys); //create a list from keys[]
		
		Collections.sort(l); //sort the list

		i = 0;
		while (i < l.size())
		{
			
			String key = l.get(i);
			//prints "key" and "value" for every key in list l
			System.out.println(key + "\t - \t" + statsMap.get(key) );
			i++;
		}
	}
	
	/**
	 * This method prints a xmlDocument on standard-out
	 * @param xmlDocument
	 */
	private void printXMLDoc(Document xmlDocument)
	{
		
		try
		{
			transformerfactory.newTransformer().transform(new DOMSource(xmlDocument),new StreamResult(System.out));
		}
		/*catch (ParserConfigurationException e)
		{
			System.out.println("unable to create new test-document\n");
			e.printStackTrace();
			System.exit(1);
		}*/
		catch (TransformerConfigurationException e)
		{
			System.out.println("transformer config error\n");
			e.printStackTrace();
			System.exit(1);
		} 
		catch (TransformerException e)
		{
			System.out.println("transformer error\n");
			e.printStackTrace();
			System.exit(1);
		}
	}
}
